// ======================================================================
//
// ControlMessageQueue.h
// jeff grills
//
// copyright 1997 origin systems inc
//
// ======================================================================

#ifndef CONTROL_MESSAGE_QUEUE_H
#define CONTROL_MESSAGE_QUEUE_H

// ======================================================================

#include "ControlMessageQueue.def"

// ======================================================================

struct ControlMessageQueueEntry
{
	ControlMessage message;
	real           value;
	void          *data;
};

// This is a circular queue with an iterator.
// A marker must be placed to stop the iterator 
// from progressing beyond it at the end of a frame, and
// reset() will clear the queue up to the marker.
class ControlMessageQueue
{
private:

	bool         readLock;
	bool         writeLock;

	int          queueMax;
	int          queueStart;      // this points to the FIRST element that is used
	int		     queueEnd;        // this points to the NEXT element to be used
	mutable int  queueIterator;   // this points to the element currently being looked at (reset moves to queueStart)
	int          queueMarker;
	ControlMessageQueueEntry *queue;

private:
	
	ControlMessageQueue(void);
	ControlMessageQueue(const ControlMessageQueue &);
	ControlMessageQueue &operator =(const ControlMessageQueue &);

public:

	explicit ControlMessageQueue(int argQueueMax);
	~ControlMessageQueue(void);

	// reset routines
	void   resetToMarker(void);
	void   setMarker(void);

	// add message routine
	void   event(ControlMessage message, real value, void *data=NULL);

	// copy the contents of the specified queue 
	void   copyToMarker(const ControlMessageQueue *copyMe);
	void   appendToMarker(const ControlMessageQueue *copyMe);

	// queue based routines
	void   resetIterator(void) const;       // this uses a MUTABLE iterator
	void   incrementIterator(void) const;   // this uses a MUTABLE iterator
	bool   atEnd(void) const;
	void   getCurrentEntry(ControlMessage *message, real *value, void **data=NULL) const;

	void   eraseEntry(void);  // marks whatever this entry is as deleted, to be skipped from now on

	void   lockRead(void);
	void   unlockRead(void);
	void   lockWrite(void);
	void   unlockWrite(void);
};

// ======================================================================

#endif
